/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is written by Juan Lang.
 *
 * The Initial Developer of the Original Code is 
 * Juan Lang.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "nsString.h"
#include "nsIFile.h"
#include "nsDirectoryServiceDefs.h"
#include "dhcpHelper.h"

#define WPAD_CMD "dhcpwrapper"

static nsresult
getOutputFromHelper(nsACString *aOutputString)
{
    if (!aOutputString) return NS_ERROR_NULL_POINTER;

    nsresult rv;
    nsCOMPtr<nsIFile> file;

    rv = NS_GetSpecialDirectory(NS_XPCOM_CURRENT_PROCESS_DIR,
     getter_AddRefs(file));
    NS_ENSURE_TRUE(NS_SUCCEEDED(rv), rv);

    rv = file->AppendNative(nsDependentCString(WPAD_CMD));
    NS_ENSURE_TRUE(NS_SUCCEEDED(rv), rv);

    PRBool canExecute = PR_FALSE;
    rv = file->IsExecutable(&canExecute);
    NS_ENSURE_TRUE(NS_SUCCEEDED(rv), rv);
    if (canExecute) {
        nsCAutoString helperNativePath;

        rv = file->GetNativePath(helperNativePath);
        NS_ENSURE_TRUE(NS_SUCCEEDED(rv), rv);

        FILE *pipe = popen(helperNativePath.get(), "r");

        NS_ENSURE_TRUE(pipe, NS_ERROR_FILE_EXECUTION_FAILED);

        char buf[80];
        int readBytes;

        while ((readBytes = fread(buf, 1, sizeof(buf), pipe)) > 0)
            aOutputString->Append(buf, readBytes);
        pclose(pipe);
    }
    return rv;
}

static nsresult
parseOutputLine(const nsCString &aLine, OptionCallback aCallback)
{
    if (!aCallback) return NS_ERROR_NULL_POINTER;

    const char *ptr = aLine.get();
    int ret, code, chars;

    nsresult rv = NS_OK;
    ret = sscanf(ptr, "%d: %n", &code, &chars);
    if (ret >= 1) {
        if (code < MAX_DHCP_OPTION) {
            PRUint8 data[MAX_DHCP_OPTION];
            PRUint8 numBytes = 0;

            while (ret > 0 && *ptr && numBytes < MAX_DHCP_OPTION) {
                int byte;

                ptr += chars;
                ret = sscanf(ptr, "%x %n", &byte, &chars);
                if (ret >= 1 && byte >= 0 && byte <= MAX_DHCP_OPTION)
                    data[numBytes++] = byte;
                else
                    ret = -1;
            }
            aCallback(code, data, numBytes);
        }
        else
            rv = NS_ERROR_INVALID_ARG;
    }
    else
        rv = NS_ERROR_INVALID_ARG;
    return rv;
}

nsresult
getDHCPOptionsFromHelper(OptionCallback aCallback)
{
    if (!aCallback) return NS_ERROR_NULL_POINTER;

    nsresult rv;
    nsCString pipeOutput;

    rv = getOutputFromHelper(&pipeOutput);
    if (NS_SUCCEEDED(rv)) {
        PRInt32 offset = 0;
        do {
            PRInt32 newline = pipeOutput.FindChar('\n', offset);

            if (newline != -1) {
                nsCAutoString line;

                pipeOutput.Mid(line, offset, newline);
                line.SetCharAt('\0', newline - offset + 1);
                parseOutputLine(line, aCallback);
                offset = newline + 1;
            }
            else
                offset = -1;
        } while (offset != -1);
    }
    return rv;
}
