/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is written by Juan Lang.
 *
 * The Initial Developer of the Original Code is 
 * Juan Lang.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#ifndef __DHCPINFORM_H__
#define __DHCPINFORM_H__

#include "nspr.h"

#ifdef __cplusplus
extern "C" {
#endif

/* This section of the header deals with parsing DHCP packets. */

/* The max value of any option */
#define MAX_DHCP_OPTION 255

/* function prototype for the option parsing function.  Will be called once
 * per encountered option with the option's code (code), as well a pointer to
 * the option's data and a value representing the maximum length of the option.
 */
typedef void (*OptionCallback)(PRUint8 code, const PRUint8 *option,
 PRUint16 len);

/* This function sends DHCPINFORM requests on every interface in interfaces
 * for which the sock member is not zero.  It then waits for up to 5 seconds
 * for a response.  If a response is received that matches the request, calls
 * callback once for every DHCP option it receives.
 * It returns after either 5 seconds have elapsed, or a response which matched
 * the sent request is received.
 * Returns 0 on success, -1 on failure.  Success in this case means we sent
 * the request on each interface and we either timed out or successfully parsed
 * at least one response.
 */
int doDHCPInform(struct InterfaceInfo *interfaces, OptionCallback callback);

#ifdef XP_UNIX
/* UNIX-specific function to "drop permissions"--literally, change the
 * effective UID to the real UID.
 */
void dropPermissions(void);

/* UNIX-specific function to "raise permissions"--literally, change the
 * effective user ID to 0 (root).
 */
void raisePermissions(void);
#else
#define dropPermissions()
#define raisePermissions()
#endif /* XP_UNIX */

#ifdef __cplusplus
}
#endif

#endif /* ndef __DHCPINFORM_H__ */
